# -*- coding: utf-8 -*-
import inspect
from ..utils.BeanUtil import BeanUtil
import re
"""
 * bean 反射器
 *<B>说明：</B>
 *<pre>
 *  略
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class ClassReflection:

    BEAN_REF_REGEX = r'<ref::([^>]+)?>'
    PARAMS_REGEX = r'<(.+)::([^>]+)?>'
    PARAMS_SPLIT_CHARACTER = '|';

    ARGS_DICT_ALIAS = '-**-';

    # 构造器
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param clazz 目标类名
    def __init__(self,clazz):

        # 目标类
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.clazz = clazz

        # 目标类的构造参数
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.parameters = {}

        # 构造器默认参数
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.defaultArgsDict = {}

        # 构造器默认参数序号index
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.defaultArgsIndexMap = []


    # 创建对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param clazz 目标类名
    def make(self,args):
        beanClazzMeta = BeanUtil.getModuleMeta(self.clazz)
        params = self.buildParmas(args);
        beanArgs = params[0];
        beankwArgs = params[1];

        return  beanClazzMeta(*beanArgs,**beankwArgs)


    # 合并构造参数
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param args 外部构造参数
    # :param params 默认构造参数
    def mergeArgs(self, beanArgs = [],defaultArgs = {}):

        from .Definition import Definition

        indexArgs = beanArgs[0]
        dictArgs = beanArgs[1]
        if not dictArgs:
            dictArgs = {};



        newDictArgs = {}
        newDictArgs.update(defaultArgs)
        newDictArgs.update(dictArgs)

        kwArgs = [];

        if indexArgs:
            for index in range(len(indexArgs)):
                name = self.defaultArgsIndexMap[index]
                if newDictArgs[name] == self.__class__.ARGS_DICT_ALIAS:
                    newDictArgs.update(indexArgs[index])
                else:
                    kwArgs.append(indexArgs[index]);
                newDictArgs.pop(name)

        clazzDictArgs = {};
        if newDictArgs:
            for name in newDictArgs:
                if newDictArgs[name] == self.__class__.ARGS_DICT_ALIAS:
                    continue;

                defaultValue = newDictArgs[name]
                if isinstance(defaultValue,Definition):
                    clazzDictArgs[name] = defaultValue.make([])
                else:
                    clazzDictArgs[name] = defaultValue


        return [kwArgs,clazzDictArgs]



    # 获取构造方法参数
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def getDefaultArgs(self):

        if self.defaultArgsDict:
            return self.defaultArgsDict

        beanClassMeta = BeanUtil.getModuleMeta(self.clazz)

        signature = inspect.signature(beanClassMeta)
        defaultArgs = signature.parameters;

        for name, value in defaultArgs.items():

            if isinstance(value.default,str):
                self.defaultArgsDict[name] = self.getDefaultValue(value.default)
            elif str(value).find("**") != -1:
                self.defaultArgsDict[name] = self.__class__.ARGS_DICT_ALIAS
            else:
                self.defaultArgsDict[name] = value.default

            self.defaultArgsIndexMap.append(name)


        return self.defaultArgsDict

    # 构建真实默认构造参数
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def buildParmas(self,args = []):

        args = self.mergeArgs(args,self.getDefaultArgs())

        return args

    @classmethod
    def getDefaultValue(cls, value):

        from .Definition import Definition
        refMatch = re.match(cls.BEAN_REF_REGEX, value)
        if refMatch:
            ref = refMatch.group(1);
            definition = Definition(_ref = ref)
            value = definition
        else:
            funcMatch = re.match(cls.PARAMS_REGEX, value)
            if funcMatch:
                funcName = funcMatch.group(1);
                funcParams = funcMatch.group(2)
                funcMeta = BeanUtil.getModuleMeta(funcName)
                funcParamstuple = tuple(funcParams.split(cls.PARAMS_SPLIT_CHARACTER))
                definition = Definition(_func = [funcMeta,funcParamstuple])
                value = definition

        return value







# -*- coding: utf-8 -*-

from .base.Container import Container
from .base.Definition import Definition
import inspect

# 标识类为bean组件装饰器
# <B> 说明： </B>
# <pre>
# 略
# </pre>
def id(beanId = '',**definition):

    def decorator(bean):
        clazzModule = inspect.getmodule(bean).__name__;
        clazzBean = clazzModule + '.' + bean.__name__
        beanConf = {
            'clazz': clazzBean,
        }

        if beanId:
            beanConf['id'] = beanId
        beanConf.update(definition)
        BeanManager.registerBeanComponent(clazzBean,beanConf)

        return bean


    return decorator

# 标识类属性为其他bean组件对象装饰器
# <B> 说明： </B>
# <pre>
# 略
# </pre>
def ref(beanId,**beanConf):

    def decorator(property):
        clazzModule = inspect.getmodule(property).__name__
        clazzBean = clazzModule + '.' + property.__qualname__.split('.')[0];
        propertyName = property.__name__;
        beanConf['clazz'] = clazzBean
        realName = beanConf.pop('name',None)
        if realName:
            beanConf[realName] = '<ref::'+beanId+'>'
        else:
            beanConf['__' + propertyName] = '<ref::' + beanId + '>'

        BeanManager.registerBeanComponent(clazzBean, beanConf)

        return property

    return decorator

# 创建容器管理器对象
# <B> 说明： </B>
# <pre>
# 略
# </pre>
def make()->'BeanManager':

    return BeanManager.make()



"""
 * 容器管理器
 *<B>说明：</B>
 *<pre>
 *  略
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  bean 组件基本配置
    components = {

        'user':{
            'clazz': 'site.service.User.User',
            '_args':[
                '你想什么'
            ],
        },
        
        'addressuser':{
            'clazz':'site.service.Address.Address',
            'addr':'xxxxx',
            'user':'<ref::user>'
        },

        'address':{
            'clazz':'site.service.Address.Address',
            'addr':'xxxxx',
            'user':'<hehe.common.func.hbean::user>'
        }
    }
 
    address = he.getBean("address")
    
    user = he.getBean("user")
    
    
    常用格式
    
    {
        'class'：'site.service.User.User',// 类路径
        '_single':True, // 是否单例 默认是单例,
       '_scope':'app',// 对象作用域,app 应用级别，forever 永远不失效
       '_init':'init',// 初始化方法,对象创建后调用设置的方法(设置属性完成后调用)
        '_args':[], // 构造方法参数，支持索引，关联数组
        '_attrs':[] // 类其他属性，直接注入
       'attr1'：'26',
       'attr2'：'',
       'attrBean'：'<func::user>',属性调用函数,自动调用func函数,函数参数为account
       'account'：'<ref::account|1>'
       'name57'：Definition([
       '_ref'：'account',// name57 为另一个bean 为account 的对象
       ]),
  }
 
 
 class Address:
    addr = '地址市什么柜子'
    user = None
    def __init__(self,user:User='<ref::user>',address='address'):
        self.addr = address
        self.user = user;
        
    def getName(self,cond):

        return 'address:' + self.addr + str(random.randint(0,10))
    
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class BeanManager():

    SCOPE_REQUEST = 'request';

    CLASS_KEY_NAME = "clazz";

    # 当前对象
    # <B> 说明： </B>
    # <pre>
    # app 应用级别,应用启动,开始生效
    # request 请求级别,每次请求时生效,请求结束后失效
    # </pre>
    manager = None;

    # 类bean定义
    # <B> 说明： </B>
    # <pre>
    # 通过类装饰器定义的bean 配置,以类路径为key
    # </pre>
    clazzBeanComponents = {}


    # 构造器
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def __init__(self,components = {}):
        """ 定义属性　"""

        # 容器列表
        # <B> 说明： </B>
        # <pre>
        # app 应用级别,应用启动,开始生效
        # request 请求级别,每次请求时生效,请求结束后失效
        # </pre>
        self.scopeContainer = {}

        # bean定义对象列表
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.definitions = {}

        # bean 组件配置
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.components = components

        # 类与bend id 的对应关系
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.clazzBeanIdMap = {}

        # 初始化参数
        BeanManager.manager = self;

        # 注册组件
        self.batchRegister(self.formatBeanComponent())

        # 范围容器事件
        self.scopeEvent = {};

    @classmethod
    def getBeanManager(cls)->'BeanManager':

        return cls.manager

    # 创建容器管理对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def make(cls,components = {})->'BeanManager':

        return cls(components);


    def setScopeEvent(self,**scopeEvent):

        self.scopeEvent = scopeEvent;

        return ;
    # 创建容器管理对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def makeContainer(self,scope):

        return Container(scope);

    # 获取指定应用对应的容器对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param scope 应用级别
    def getScopeContainer(self,scope)->Container:

        scopeContainerFunc = self.scopeEvent.get(scope,None)

        if scopeContainerFunc is None:
            container = self.scopeContainer.get(scope,None);
            if container is None:
                container = Container(scope);
                self.scopeContainer[scope] = container
        else:
            container = scopeContainerFunc();

        return container

    # 清理容器
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def cleanScopeContainer(self,scopes = []):

        if scopes.count() == 0 :
            scopes.append(self.SCOPE_REQUEST)

        for scope in scopes:
            self.scopeContainer.pop(scope)


    # 获取bean 实例
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    # :param args 对象参数
    def getBean(self,beanId,*args ,**kwargs):

        definition = self.getDefinition(beanId)
        container = definition.getContainer()
        bean = None
        if container.hasBean(beanId) :
            bean = container.getBean(beanId)
        else:
            bean = definition.make([args,kwargs])
            if definition.isSingle():
                container.setBean(beanId,bean)

        return bean

    # 是否定义bean
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    # :param args 对象参数
    def hasComponent(self, beanId):

        component = self.components.get(beanId, None)
        if component is not None:
            return True
        else:
            return False

    # 创建一个新的bean对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    # :param args 对象参数
    def makeBean(self, beanId, *args ,**kwargs):

        definition = self.getDefinition(beanId)
        bean = definition.make([args,kwargs])

        return bean

    # 获取bean 定义对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    def getDefinition(self,beanId)->Definition:


        definition = self.definitions.get(beanId,None)
        if definition:
            return definition;

        component = self.components.get(beanId,None)
        if component is None:
            component = {self.CLASS_KEY_NAME: beanId}


        self.definitions[beanId] = Definition(**component)

        return self.definitions[beanId]

    # 批量注册bean组件
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    def batchRegister(self,components = {}):

        self.components.update(components)

    # 追加配置
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    # :param component bean 配置信息
    def appendComponent(self,beanId = '', component={}):

        return ;

    # 获取组件配置
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    def getComponents(self):

        return self.components

    # 注册装饰器bean 配置定义
    # <B> 说明： </B>
    # <pre>
    # 注入的属性自动叠加
    # </pre>
    @classmethod
    def registerBeanComponent(cls,clazz,bean):

        beanComponent = cls.clazzBeanComponents.pop(clazz,None)
        if beanComponent is None:
            beanComponent = bean
        else:
            beanComponent.update(bean)

        cls.clazzBeanComponents[clazz] = beanComponent;

        if cls.manager is not None:
            component = cls.formatComponentForBean(clazz,beanComponent)
            cls.manager.batchRegister(component);

        return ;


    @classmethod
    def formatComponentForBean(self,clazz,beanComponent):

        beanId = beanComponent.pop('id', None)
        if not beanId:
            beanId = clazz
        else:
            beanComponent['id'] = beanId;

        component = {};
        component[beanId] = beanComponent

        return component;

    @classmethod
    def formatBeanComponent(cls):

        componentList = {};
        for clazz in cls.clazzBeanComponents:
            beanComponent =  cls.clazzBeanComponents[clazz]
            beanId = beanComponent.pop('id',None)
            if not beanId:
                beanId = clazz
            else:
                beanComponent['id'] = beanId;

            componentList[beanId] = beanComponent

        return componentList



# -*- coding: utf-8 -*-
import importlib,inspect,traceback
"""
 * 类帮助类
 *<B>说明：</B>
 *<pre>
 *  提供基本类操作,获取属性值,设置属性等等
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class BeanUtil:

    BEAN_REF_REGEX = r'<ref::([^>]+)?>'
    PARAMS_REGEX = r'<(.+)::([^>]+)?>'

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def getAttrs(cls,object):
        attrs = dir(object);
        attrDict = {};
        for attr in attrs:
            if not attr.startswith("__"):
                attrDict[attr] = getattr(object,attr)

        return attrDict

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def setAttrs(cls,object,attrDict = {}):
        for attr in attrDict:
            setattr(object, attr, attrDict[attr])

    def splitClassName(clazz):

        packageClass = clazz.rsplit('.', 1)

        return [packageClass[0],packageClass[1]]

    # 获取类或对象的自定义属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def getModuleMeta(cls,clazz):

        packageClass = clazz.rsplit('.', 1)
        packageMeta = importlib.import_module(packageClass[0])
        #
        return  getattr(packageMeta, packageClass[1])


# -*- coding: utf-8 -*-


"""
 * 容器类
 *<B>说明：</B>
 *<pre>
 *  略
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class Container:

    # 构造器
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param scope 应用级别
    def __init__(self,scope):

        # 容器有效范围
        # <B> 说明： </B>
        # <pre>
        # app 应用级别，forever 永远不失效
        # </pre>
        self.scope = scope

        # bean 对象列表
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.beans = {}


    # bean是否存在
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    def hasBean(self,beanId = '')->bool:

        if self.beans.get(beanId,None) is None:
            return False
        else:
            return True


    # 获取 bean 对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param id bean id
    def getBean(self,beanId = ''):

        return self.beans.get(beanId,None)

    # 设置bean对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    # :param beanId bean id
    # :param bean bean 对象
    def setBean(self,beanId,bean = object):

        self.beans[beanId] = bean

        return ;




# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open("./README.md", "r",encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name = 'hehey-hcontainer',
    version = '1.0.1',
    author = '13564768842',
    packages=find_packages(),
    author_email = 'chinabluexfw@163.com',
    url = 'https://gitee.com/chinahehe/hehey-hcontainer',
    description = 'hehey-hcontainer 是一个python di 容器,提供依赖注入等等功能',
    long_description=long_description,
    long_description_content_type="text/markdown",
)


# -*- coding: utf-8 -*-
from .Container import Container
from .ClassReflection import ClassReflection
import re
from ..utils.BeanUtil import BeanUtil

"""
 * bean 定义类描述
 *<B>说明：</B>
 *<pre>
 *  略
 *</pre>
 *<B>示例：</B>
 *<pre>
 *  略
 *</pre>
 *<B>日志：</B>
 *<pre>
 *  略
 *</pre>
 *<B>注意事项：</B>
 *<pre>
 *  略
 *</pre>
"""
class Definition():

    BEAN_REF_REGEX = r'<ref::([^>]+)?>'
    PARAMS_REGEX = r'<(.+)::([^>]+)?>'
    PARAMS_SPLIT_CHARACTER = '|';

    # 系统默认属性
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    sysAttr = ['_attrs', '_scope', '_ref','_func', 'clazz', '_single', '_init', '_args', '_onProxy', '_proxyHandler'];

    def __init__(self,id = '' ,**attrs):

        # 容器全局唯一id
        # <B> 说明： </B>
        # <pre>
        # 如未设置,则默认为对象的类名
        # </pre>
        self._id = "";

        # 其他bean id
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self._ref = None;

        # 方法
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self._func = [];

        # 配置基础
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.clazz = None;

        # 构造方法参数，支持索引，关联数组
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self._args = [];

        # 类其他属性，直接注入
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self._attrs = [];

        # 作用范围
        # <B> 说明： </B>
        # <pre>
        # app 应用级别(每次请求结束后自动销毁)
        # forever 永久级别(必须重启php 服务后才能自动销毁)
        # </pre>
        self._scope = 'app';

        # 反射对象
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self.reflection = None;

        # 初始化参数
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self._init = None;

        # 是否单例
        # <B> 说明： </B>
        # <pre>
        # 略
        # </pre>
        self._single = True

        self.__formatArgs = None

        self.__formatAttrs = None
        self._id = id;

        if attrs:
            befinitionAttrs = self.formatAttributes(attrs);
            BeanUtil.setAttrs(self, befinitionAttrs)

        return ;

    def getId(self):

        return self._id

    def getRef(self):

        return self._ref

    def getClazz(self):

        return self.clazz

    def setSingle(self,single = True):

        self._single = single

    def isSingle(self):

        return self._single


    def getContainer(self)->'Container':

        from ..bean import BeanManager
        return BeanManager.getBeanManager().getScopeContainer(self._scope)


    # 创建bean 对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def make(self,args):

        from ..bean import BeanManager

        if self._ref is not None:
            return BeanManager.getBeanManager().getBean(self._ref)
        elif self._func:
            funcMeta =  self._func[0];
            funcParamstuple = self._func[1];
            return funcMeta(*funcParamstuple);
        else:
            # 获取反射类
            reflection = self.getReflection()
            # 构建器参数
            parameters = self.buildArgs(args);
            # 创建对象
            beanObj = reflection.make(parameters)

            # 设置属性
            if self._attrs :
                self.buildAttrs()
                BeanUtil.setAttrs(beanObj,self.__formatAttrs)

            # 调用初始化方法
            if self._init:
                initFuncMeta = getattr(beanObj,self._init)
                initFuncMeta()

            return beanObj


    # 构建类的构造参数(args)
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def buildArgs(self,args):

        if self.__formatArgs is None:
            self.__formatArgs = self.formatArgs(self._args)

        beanArgs = []
        beanArgs = beanArgs + self.__formatArgs
        userArgs = args[0]
        if len(beanArgs) >= len(userArgs):
            if len(userArgs) == 1 and not userArgs[0]:
                pass
            else:
                for i in range(len(userArgs)):
                    beanArgs[i] = userArgs[i]

            args[0] = beanArgs;

        return args

    # 构建类属性(attrs)
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def buildAttrs(self):

        if self.__formatAttrs is None:
            self.__formatAttrs = self.formatAttrs(self._attrs)

        return ;

    # 获取反射对象
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    def getReflection(self):

        if self.reflection is None:
            self.reflection = ClassReflection(self.clazz)

        return self.reflection

    # 整理类自定义参数
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def formatAttributes(cls,attrs = {}):

        attributes = {}
        customAttrs = {}
        for attr in attrs:
            if attr in cls.sysAttr:
                attributes[attr] = attrs[attr]
            else:
                customAttrs[attr] = attrs[attr]

        _argsStatus = attributes.get('_args',None)
        if _argsStatus is True:
            attributes['_args'] = [customAttrs]
        else:
            attributes['_attrs'] = customAttrs

        return attributes

    # 构建参数
    # <B> 说明： </B>
    # <pre>
    # 略
    # </pre>
    @classmethod
    def formatAttrs(cls,attrs = {}):

        if not attrs :
            return attrs

        newAttrs = {};
        for key in attrs:
            value = attrs[key]
            if key[0] == '@':
                newAttrs[key[1:]] = cls.formatBean(value,True)
            else:
                newAttrs[key] = cls.formatBean(value)

        return newAttrs

    @classmethod
    def formatArgs(cls,args = []):

        if not args :
            return args

        if isinstance(args,dict):
            return [args]

        for index in range(len(args)):
            args[index] = cls.formatBean(args[index])

        return args

    @classmethod
    def formatBean(cls,value,ergodic = False):

        if ergodic:
            if isinstance(value,dict):
                for key in value:
                    value[key] = cls.getBeanValue(value[key])
            elif isinstance(value,list):
                for index in range(len(value)):
                    value[index] = cls.getBeanValue(value[index])
        else:
            if type(value) == cls:
                value = value.make([])
            else:
                if type(value) == str:
                    value = cls.getBeanValue(value)

        return value

    @classmethod
    def getBeanValue(cls, value):

        refMatch = re.match(cls.BEAN_REF_REGEX, value)
        if refMatch:
            ref = refMatch.group(1);
            definition = Definition(_ref = ref)
            value = definition.make([])
        else:
            funcMatch = re.match(cls.PARAMS_REGEX, value)
            if funcMatch:
                funcName = funcMatch.group(1);
                funcParams = funcMatch.group(2)
                funcMeta = BeanUtil.getModuleMeta(funcName)
                funcParamstuple = tuple(funcParams.split(cls.PARAMS_SPLIT_CHARACTER))

                definition = Definition(_func = [funcMeta, funcParamstuple])
                value = definition

        return value








